require 'airrecord'
require 'active_support/core_ext/string/inflections'

module AppRail
  module Airtable
    class ApplicationRecord < Airrecord::Table
      include ActiveSupport::Inflector
    
      def self.base_key
        ENV.fetch("AIRTABLE_BASE_ID")
      end
      
      def self.table_name
        self.name.pluralize
      end
    
      def self.airtable_attr(*attributes)
        attributes.each do |attribute|
          define_method(attribute.to_s.snake_case) { self[attribute] }
          define_method("#{attribute.to_s.snake_case}=") { |value| self[attribute] = value }
        end
      end
    
      # Step utilities
      def self.ar_list_item(text:, detail_text: nil, image: nil, sf_symbol: nil, material_icon: nil)
        define_method(:ar_list_item_as_json) do
          {
            id: self.id, 
            text: method_value(text).to_s, 
            detailText: method_value(detail_text).to_s, 
            imageURL: method_value(image), 
            sfSymbolName: method_value(sf_symbol), 
            materialIconName: method_value(material_icon)
          }.compact
        end
      end
      
      def self.ar_stack(text_items:)
        define_method(:ar_stack_as_json) do
          text_items.map{|ti| {type: :text, label: ti, text: self[ti].to_s} }
        end
      end
      
      # Customisable behaviour
      
      # Override to provide custom sorting or filtering for index
      def self.index(user:)
        all
      end
      
      private
      
      def method_value(name)
        send(name) if name.present?
      end
      
      # size is either :small, :large or :full
      def image(name, index: 0, size: :full)
        self[name][index]["thumbnails"][size.to_s]["url"] if self[name] && self[name].length > index
      end
      
    end
  end
end