require 'bcrypt'
require 'securerandom'

module AppRail
  module Airtable
    module Authenticatable
      include BCrypt
  
      def self.included(klass)
        klass.extend(ClassMethods)
        klass.prepend(InstanceMethods) # prepends `initialize` instance method in order to take precedence over Airrecord::Table#initialize
      end
      
      module ClassMethods
        def create(email:, password:)
          user = new("Email" => email, "Password Hash" => password_hash(password), "Access Token" => next_access_token)
          user.create
          user
        end
        
        def create_session_as_json(email:, password:)
          user = find_by_email_and_password(email, password)
          return nil unless user
          
          user["Access Token"] = next_access_token
          user.save
          user&.oauth_session
        end
  
        def find_by_email_and_password(email, password)
          user = all(filter: "{Email} = \"#{email}\"").first
          user&.valid_password?(password) ? user : nil
        end

        def find_by_access_token(access_token)
          all(filter: "{Access Token} = \"#{access_token}\"").first 
        end
        
        def password_hash(password)
          BCrypt::Password.create(password)
        end   
        
        def next_access_token
          SecureRandom.hex
        end    
      end

      module InstanceMethods
        def initialize(*args)
          if (kwargs = args.first) && kwargs.is_a?(Hash) && kwargs.size == 2 && kwargs.key?(:email) && kwargs.key?(:password)
            super("Email" => kwargs[:email], "Password Hash" => self.class.password_hash(kwargs[:password]), "Access Token" => self.class.next_access_token)
          else
            super
          end
        end
      end

      def valid_password?(password)
        BCrypt::Password.new(password_hash) == password
      end
      
      def password_hash
        self["Password Hash"]
      end
    
      def oauth_session
        { access_token: self["Access Token"], scope: :user, refresh_token: "", token_type: :bearer, expires_in: 60000 }
      end
    end
  end
end