# frozen_string_literal: true

require 'active_support/core_ext/string/inflections'
require 'app_rail/airtable/string_ext'
require 'thor'
require 'airrecord'

module AppRail
  module Airtable
    class Generator < Thor::Group
      include Thor::Actions

      argument :output_directory
      argument :api_key
      argument :base_id
      argument :schema, required: false

      def self.source_root
        File.join(File.dirname(__FILE__), '..', '..', '..')
      end

      def create_project
        validate_airtable_schema
        directory('templates/project', output_directory)
      end

      def self.exit_on_failure?
        true
      end

      private

      def validate_airtable_schema
        tables.each do |table|
          table_name = table['name']
          table_fields = table['fields'].concat(table['associations']).map { |field| field['name'] }.compact

          airtable_table = find_table(table_name)
          compare_fields(table_fields, airtable_table)
        end
      end

      def tables
        @tables ||= schema ? JSON.parse(schema)['tables'] : []
      end

      def find_table(table_name)
        Airrecord.table(api_key, base_id, table_name)
      end

      def compare_fields(table_fields, airtable_table)
        airtable_table.records.first.fields.each_key do |key|
          next if table_fields.include? key

          raise "ERROR! The key \"#{key}\" is not present in the schema definition."
        end
      end

      def fields_as_params_examples(fields)
        fields.each_with_object({}) do |field, hsh|
          hsh[field['name']] = case field['type']
                               when 'date'
                                 '01/01/2022'
                               when 'string', 'text'
                                 'MyString'
                               when 'integer'
                                 10
                               else
                                 'MyString'
                               end
        end
      end
    end
  end
end
