require 'sinatra'
require 'json'
require 'active_support'

# TODO: MBS - move to configure block or other
Airrecord.api_key = ENV.fetch("AIRTABLE_API_KEY")

class Symbol
  
  def pluralize
    ActiveSupport::Inflector.pluralize(self)
  end
  
  def classify
    ActiveSupport::Inflector.classify(self)  
  end
  
  def classify_constantize
    ActiveSupport::Inflector.constantize(classify)
  end
  
end

module AppRail
  module Airtable
    class Sinatra < Sinatra::Base
      @@authenticated_route = false
      
      helpers do
        def request_body_as_json
          request.body.rewind
          JSON.parse(request.body.read)
        end
        
        def params_and_body_as_json
          request_body_as_json.merge(params)
        end
      end
      
      def self.authenticatable_resources(name, only: [:index, :show, :create, :update])
        
        # If authentication is used then include the correct helpers
        # Allowing the routes to use `authenticate!` and `current_user`
        helpers AppRail::Airtable::AuthenticationHelpers
        
        resources(name, only: only)
        
        return unless block_given?
        @@authenticated_route = true
        yield
        @@authenticated_route = false
      end

      def self.resources(name, only: [:index, :show, :create, :update])
        only = [only] if only.is_a?(Symbol)

        index_route(name, authenticated_route?) if only.include?(:index)
        show_route(name, authenticated_route?) if only.include?(:show)
        create_route(name, authenticated_route?) if only.include?(:create)
        update_route(name, authenticated_route?) if only.include?(:update)
      end
      
      def self.index_route(name, authenticated_route)
        get "/#{name.to_s}" do
          authenticate! if authenticated_route
          name.classify_constantize.index(user: authenticated_route ? current_user : nil).map(&:ar_list_item_as_json).to_json
        end
      end
      
      def self.show_route(name, authenticated_route)
        get "/#{name.to_s}/:id" do
          authenticate! if authenticated_route
          name.classify_constantize.find(params['id']).ar_stack_as_json.to_json
        end
      end
      
      def self.create_route(name, authenticated_route)
        post "/#{name.to_s}" do
          authenticate! if authenticated_route
          as_json = name.classify_constantize.create_as_json(current_user: authenticated_route ? current_user : nil, params: params_and_body_as_json)
          [201, as_json.to_json] 
        end
      end

      def self.update_route(name, authenticated_route)
        put "/#{name.to_s}/:id" do
          authenticate! if authenticated_route
          record = name.classify_constantize.find(params['id'])
          as_json = record.update_as_json(current_user: authenticated_route ? current_user : nil, params: params_and_body_as_json)
          [200, as_json.to_json]
        end
      end

      def self.authenticated_route?
        @@authenticated_route
      end
    end
  end
end